import type { Immutable, TypedArray } from '../../types/type';
import type { Geometry, GeometryOptions, AttributeWithType, Attribute } from '../../types/Geometry';
import type { GPUBuffer, GPUBufferOptions } from '../../types/GPUBuffer';
import { GLGeometry } from '../webgl/GLGeometry';
import type { MarsRenderer } from './MarsRenderer';
type BufferDirtyFlag = {
    dirty: boolean;
    discard?: boolean;
    start?: number;
    end?: number;
};
/**
 * 应用层Geometry对象，本身不直接保存GPU资源而是通过geometryInternal成员保存GPU资源。
 *
 */
export declare class MarsGeometry implements Geometry {
    renderer?: MarsRenderer;
    _buffers: {
        [key: string]: GPUBufferOptions;
    };
    _attributes: Record<string, AttributeWithType>;
    _attributesReleasable: {
        [key: string]: boolean;
    };
    _attributesName: string[];
    _dirtyFlags: {
        [key: string]: BufferDirtyFlag;
    };
    _index?: Uint16Array | Uint32Array;
    _indexReleasable: boolean;
    readonly internal: GLGeometry;
    readonly options?: GeometryOptions;
    readonly name: string;
    protected _drawCount: number;
    protected _drawStart: number;
    protected _mode: number;
    protected _isDestroyed: boolean;
    indexDataType: Geometry['indexDataType'];
    get isDestroyed(): boolean;
    get mode(): number;
    set mode(n: number);
    get attributes(): Immutable<{
        [key: string]: Attribute;
    }>;
    get drawCount(): number;
    set drawCount(n: number | undefined);
    get drawStart(): number;
    set drawStart(n: number);
    constructor(option: GeometryOptions);
    destroy(): void;
    setAttributeBuffer(name: string, buffer: GPUBuffer): void;
    getAttributeBuffer(name: string): GPUBuffer | undefined;
    setAttributeData(name: string, data: TypedArray): void;
    getAttributeData(name: string): TypedArray | undefined;
    setAttributeSubData(name: string, offset: number, data: TypedArray): void;
    getIndexData(): Uint16Array | Uint32Array | undefined;
    setIndexData(data: Uint16Array | Uint32Array | undefined): void;
    setIndexSubData(offset: number, data: TypedArray): void;
    getAttributeStride(name: string): number;
    getAttributeDataLength(name: string): number;
    getAttributeNames(): string[];
    assignRenderer(renderer: MarsRenderer): MarsGeometry;
    flush(): boolean;
}
export {};
