import type { MaterialDataBlock, MaterialDataBlockDestroyOptions, MaterialDataBlockOptions, UniformSemantic, UniformValue, vec2, Texture, SemanticFunc } from '../../types';
import type { MarsRenderer } from './MarsRenderer';
import { MarsTexture } from './MarsTexture';
import type { UniformBlockSpec } from '../webgl/GLUniformUtils';
import { UniformBlockBuffer } from '../webgl/GLUniformUtils';
export declare class MarsMaterialDataBlock implements MaterialDataBlock {
    readonly _uniformValues: Record<string, UniformValue>;
    readonly _uniformTextures: Record<string, MarsTexture | MarsTexture[]>;
    readonly _uniformFlags: Record<string, boolean>;
    readonly _uniformValueRanges: Record<string, vec2>;
    readonly name: string;
    readonly uniformSemantics: {
        [key: string]: UniformSemantic | SemanticFunc;
    };
    readonly renderer?: MarsRenderer;
    readonly uboBufferMap: Record<string, UniformBlockBuffer>;
    protected _isDestroyed: boolean;
    private _uniformPrivate;
    private readonly _keepUboData;
    constructor(props: MaterialDataBlockOptions & {
        keepUboData?: boolean;
    }, renderer?: MarsRenderer);
    _assignUniformValueRenderer(value: MarsTexture | MarsTexture[], renderer?: MarsRenderer): void;
    invalidAllFlags(): void;
    clearFlags(): void;
    isDirty(shaderCacheId: string, name: string): boolean;
    get isDestroyed(): boolean;
    setUniformTexture(uniformName: string, texture: MarsTexture | MarsTexture[] | undefined): void;
    getUniformTexture<T extends (Texture | Texture[]) = MarsTexture>(uniformName: string): T | undefined;
    assignRenderer(renderer: MarsRenderer): MaterialDataBlock;
    createUboBuffer(spec: UniformBlockSpec): UniformBlockBuffer | undefined;
    setUboBuffer(uboBuffer: UniformBlockBuffer): void;
    updateUniformSubData(name: string, start: number, count: number): void;
    hasUniformValue(name: string): boolean;
    destroy(options?: MaterialDataBlockDestroyOptions): void;
    getUniformValue(name: string): UniformValue;
    getUniformValues(): {
        [p: string]: UniformValue;
    };
    setUniformValue(name: string, value: UniformValue | undefined, isSemantic?: boolean): boolean;
    setUniformValues(map: {
        [p: string]: UniformValue;
    }): void;
    setUniformSemantic(name: string, semantic: string | SemanticFunc | undefined): void;
    clone(name: string): MarsMaterialDataBlock;
}
