import type { RenderPass, RenderPassAttachmentOptions, RenderPassLoadAction, RenderPassDelegate, RenderPassDepthStencilAttachment, RenderPassDestroyOptions, RenderPassOptions, RenderPassStoreAction } from '../../types/RenderPass';
import { RenderPassAttachmentStorageType, RenderPassMeshOrder } from '../../types/RenderPass';
import type { MarsMesh } from './MarsMesh';
import type { MarsRenderer } from './MarsRenderer';
import { GLFrameBuffer } from '../webgl/GLFrameBuffer';
import { MarsRenderPassColorAttachment } from './MarsRenderPassColorAttachment';
import { MarsTexture } from './MarsTexture';
import type { vec4 } from '../../types';
import { MarsSemanticMap } from './MarsSemanticMap';
export declare class MarsRenderPass implements RenderPass {
    delegate: RenderPassDelegate;
    readonly name: string;
    readonly priority: number;
    readonly meshes: MarsMesh[];
    readonly options: RenderPassAttachmentOptions;
    readonly renderer: MarsRenderer;
    readonly frameBuffer?: GLFrameBuffer;
    readonly depthStencilType: RenderPassAttachmentStorageType;
    readonly loadAction: RenderPassLoadAction;
    readonly storeAction: RenderPassStoreAction;
    readonly semantics: MarsSemanticMap;
    private _isDestroyed;
    private _depthTexture?;
    private _stencilTexture?;
    private _attachments;
    get isDestroyed(): boolean;
    get multisample(): number;
    get attachments(): MarsRenderPassColorAttachment[];
    viewportScale: number;
    private _isCustomViewport;
    private _customViewport?;
    meshOrder: RenderPassMeshOrder;
    constructor(options: RenderPassOptions, renderer?: MarsRenderer);
    get viewport(): vec4;
    get stencilAttachment(): RenderPassDepthStencilAttachment | undefined;
    get depthAttachment(): RenderPassDepthStencilAttachment | undefined;
    bind(): void;
    private _getDepthTexture;
    private _getStencilTexture;
    private _setViewportOptions;
    resetColorAttachments(colors: MarsTexture[]): void;
    resetAttachments(options: RenderPassAttachmentOptions): RenderPassAttachmentOptions;
    private _resetAttachments;
    unbind(): void;
    assignRenderer(renderer: MarsRenderer): MarsRenderPass;
    destroy(options?: RenderPassDestroyOptions): void;
    addMesh(mesh: MarsMesh): void;
    removeMesh(mesh: MarsMesh): void;
    setMeshes(meshes: MarsMesh[]): MarsMesh[];
}
