import type { GLGPURenderer } from './GLGPURenderer';
import type { RenderPassDepthStencilAttachmentOptions, RenderPassStorageObject, RenderPassStoreAction } from '../../types/RenderPass';
import { RenderPassAttachmentStorageType, RenderPassDestroyAttachmentType } from '../../types/RenderPass';
import { GLRenderBuffer } from './GLRenderBuffer';
import { GLTexture } from './GLTexture';
export interface GLFrameBufferOptions {
    attachments: GLTexture[];
    depthStencilAttachment?: RenderPassDepthStencilAttachmentOptions;
    isCustomViewport?: boolean;
    viewport: [x: number, y: number, width: number, height: number];
    viewportScale?: number;
    storeAction: RenderPassStoreAction;
    name?: string;
    multisample?: number;
}
export declare class GLFrameBuffer {
    readonly renderer: GLGPURenderer;
    readonly viewport: [x: number, y: number, width: number, height: number];
    readonly ready: boolean;
    readonly depthStencilStorageType: RenderPassAttachmentStorageType;
    readonly name: string;
    externalStorage: boolean;
    storeAction: RenderPassStoreAction;
    storeInvalidAttachments?: GLenum[];
    isCustomViewport: boolean;
    depthStencilRenderBuffer?: GLRenderBuffer;
    depthTexture?: GLTexture;
    stencilTexture?: GLTexture;
    colorTextures: GLTexture[];
    fbo?: WebGLFramebuffer;
    multisampleFbo?: WebGLFramebuffer;
    multisampleRbs: GLRenderBuffer[];
    viewportScale: number;
    readonly multisample: number;
    readonly _attachmentHandles: WebGLTexture[];
    constructor(options: GLFrameBufferOptions, renderer: GLGPURenderer);
    get stencilStorage(): RenderPassStorageObject | undefined;
    get depthStorage(): RenderPassStorageObject | undefined;
    _getHandles(): void;
    checkOptions(options: GLFrameBufferOptions): void;
    getStoreAttachments(storeAction: RenderPassStoreAction, separateDepthStencil: boolean): GLenum[] | undefined;
    unbind(): void;
    bind(): void;
    resolveMultiSample(): void;
    resetColorTextures(colors?: GLTexture[]): void;
    destroy(opt?: {
        depthStencilAttachment?: RenderPassDestroyAttachmentType;
    }): void;
    resize(x: number, y: number, width: number, height: number): void;
}
