import { GLGPUBuffer } from './GLGPUBuffer';
import { WebGLState } from './WebGLState';
import { GLShaderLibrary } from './GLShaderLibrary';
import type { GPURenderer, RendererExtensions } from '../../types';
import type { GPUBufferOptions } from '../../types/GPUBuffer';
import type { RenderFrame, RenderFrameOptions } from '../../types/RenderFrame';
import { GLGPUCapability } from './GLGPUCapability';
import { GLTexture } from './GLTexture';
import type { GLRenderBuffer } from './GLRenderBuffer';
import type { GLFrameBuffer } from './GLFrameBuffer';
export declare class GLGPURenderer implements GPURenderer {
    readonly gl: WebGLRenderingContext | WebGL2RenderingContext;
    readonly level: number;
    readonly gpu: GLGPUCapability;
    readonly state: WebGLState;
    readonly shaderLibrary: GLShaderLibrary;
    readonly emptyTexture2D: GLTexture;
    readonly emptyTextureCube: GLTexture;
    readonly _buffers: GLGPUBuffer[];
    readonly _textures: GLTexture[];
    readonly _renderBuffers: GLRenderBuffer[];
    readonly _frameBuffers: GLFrameBuffer[];
    readonly _vaos: WebGLVertexArrayObject[];
    private _sourceFbo;
    private _targetFbo;
    private _isDestroyed;
    extension: RendererExtensions;
    constructor(gl: WebGLRenderingContext | WebGL2RenderingContext);
    requestAnimationFrame(cb: FrameRequestCallback): number;
    cancelAnimationFrame(id: number): void;
    get isDestroyed(): boolean;
    onContextLose: (e: Event) => void;
    copy2(source: GLTexture, target: GLTexture): void;
    resetColorAttachments(rp: GLFrameBuffer, colors: GLTexture[]): void;
    createGLRenderBuffer(renderbuffer: GLRenderBuffer): WebGLRenderbuffer | null;
    createPipeline(options?: RenderFrameOptions): RenderFrame;
    createBuffer(options: GPUBufferOptions): GLGPUBuffer;
    resize(width: number, height: number): void;
    _assignInspectorName(obj: any, name?: string, id?: string): void;
    createGLFrameBuffer(frameBuffer: GLFrameBuffer, name?: string): WebGLFramebuffer | null;
    createGLTexture(texture: GLTexture, name?: string): WebGLTexture | null;
    createGLBuffer(gpuBuffer: GLGPUBuffer, name?: string): WebGLBuffer | null;
    deleteGLTexture(texture: GLTexture): void;
    deleteGPUBuffer(buffer: GLGPUBuffer | null): void;
    deleteGLFrameBuffer(frameBuffer: GLFrameBuffer): void;
    deleteGLRenderBuffer(renderbuffer: GLRenderBuffer): void;
    createVertexArray(name?: string): WebGLVertexArrayObject | undefined;
    _bindVertexArray(vao: WebGLVertexArrayObject | null): void;
    _bindVertexArrayOES(vao: WebGLVertexArrayObject | null): void;
    bindVertexArray(vao: WebGLVertexArrayObject | null): void;
    deleteVertexArray(vao: WebGLVertexArrayObject): void;
    destroy(haltGL?: boolean): void;
    get height(): number;
    get width(): number;
    get canvas(): HTMLCanvasElement | OffscreenCanvas;
    createRenderFrame(options?: RenderFrameOptions): RenderFrame;
}
