import type { AttributeWithType } from '../../types/Geometry';
import type { GPUBuffer, GPUBufferOptions } from '../../types/GPUBuffer';
import { GLGPUBuffer } from './GLGPUBuffer';
import type { GLGPURenderer } from './GLGPURenderer';
import type { RenderState } from '../../types/RenderFrame';
import type { GLVertexArrayObject } from './GLVertexArrayObject';
export declare const INDEX_TYPE_MAP: {
    [x: number]: number;
};
export declare const TYPE_BYTE_MAP: {
    [x: number]: number;
};
export type GLGeometryOptions = {
    drawStart: number;
    drawCount: number;
    attributes: {
        [key: string]: AttributeWithType;
    };
    buffers: {
        [key: string]: GPUBufferOptions;
    };
    mode: GLenum;
    index?: GPUBufferOptions;
    name: string;
    vaoMap?: Record<string, GLVertexArrayObject | undefined>;
};
/**
 * GLGeometry对象，保存GLGPUBuffer对象。
 *
 */
export declare class GLGeometry {
    name?: string;
    renderer?: GLGPURenderer | undefined;
    drawStart: number;
    drawCount: number;
    mode: GLenum;
    indexBufferType: GLenum;
    attributes: {
        [key: string]: AttributeWithType;
    };
    _buffersMap: {
        [key: string]: GLGPUBuffer;
    };
    _indexBuffer?: GLGPUBuffer;
    readonly vaoMap: Record<string, GLVertexArrayObject | undefined>;
    constructor(option: GLGeometryOptions | GLGeometry, renderer: GLGPURenderer);
    createVao(name: string): GLVertexArrayObject;
    destroyVao(name: string): void;
    setIndexBuffer(buffer: GLGPUBuffer, indexBufferType: GLenum): void;
    createIndexBuffer(data: Uint16Array | Uint32Array | Uint8Array, renderer: GLGPURenderer): [buffer: GLGPUBuffer, type: GLenum];
    getGPUBuffer(name: string | number): GLGPUBuffer | null;
    destroy(): void;
    setAttributeBuffer(name: string, buffer: GPUBuffer): void;
    getAttributeBuffer(name: string): GPUBuffer | null;
    draw(state: RenderState): void;
}
