import type { GLGPURenderer } from './GLGPURenderer';
import type { GLGeometry } from './GLGeometry';
import type { RenderState, SemanticFunc } from '../../types/RenderFrame';
import type { GLVertexArrayObject } from './GLVertexArrayObject';
import type { UniformBlockSpec } from './GLUniformUtils';
export interface ProgramAttributeInfo {
    readonly name: string;
    readonly size: number;
    readonly type: number;
    readonly loc: number;
}
export interface ProgramUniformInfo {
    readonly loc: WebGLUniformLocation;
    readonly subInfos: ProgramUniformInfo[];
    readonly name: string;
    readonly size: number;
    readonly type: number;
    readonly textureIndex: number;
    readonly isTexture: boolean;
}
export declare class GLProgram {
    readonly glHandle: WebGLProgram;
    readonly renderer: GLGPURenderer;
    readonly shared: boolean;
    readonly id: string;
    readonly _vaos: GLVertexArrayObject[];
    readonly uniformBlockMap: Record<string, UniformBlockSpec>;
    attrInfoMap: {
        [key: string]: ProgramAttributeInfo;
    };
    uniformInfoMap: {
        [key: string]: ProgramUniformInfo;
    };
    private _uniformBlocks;
    constructor(renderer: GLGPURenderer, program: WebGLProgram, shared: boolean, id: string);
    init(renderer: GLGPURenderer, program: WebGLProgram): void;
    bind(): void;
    getSemanticValue(state: RenderState, semanticName: string | SemanticFunc): number | Float32Array | Float64Array | Uint8Array | Uint32Array | Uint16Array | Int8Array | Int16Array | Int32Array | import("../../types").GPUBuffer | import("../../types").Texture | number[] | number[][] | import("../../types").UniformStruct | import("../../types").UniformStruct[] | SemanticFunc | import("../../types").Texture[] | null | undefined;
    setupUniforms(state: RenderState): void;
    setGLUniformValue(name: string, value: any, info: ProgramUniformInfo, gl: WebGL2RenderingContext | WebGLRenderingContext): void;
    setupAttributes(geometry: GLGeometry): GLVertexArrayObject;
    destroy(): void;
}
