import type { ShaderCompileResult, ShaderLibrary, ShaderWithSource } from '../../types/ShaderLibrary';
import type { GLGPURenderer } from './GLGPURenderer';
import { GLProgram } from './GLProgram';
interface GLShaderCompileResult extends ShaderCompileResult {
    program?: WebGLProgram;
}
declare enum ShaderType {
    vertex = 0,
    fragment = 1
}
export declare class GLShaderLibrary implements ShaderLibrary {
    readonly renderer: GLGPURenderer;
    private readonly shaderResults;
    private _shaderMap;
    private _programMap;
    private _glVertShaderMap;
    private _glFragShaderMap;
    private readonly _glAsyncCompileExt?;
    private _shaderAllDone;
    constructor(renderer: GLGPURenderer);
    get cacheIds(): string[];
    getShaderResult(cacheId: string): ShaderCompileResult | undefined;
    compileAllShaders(asyncCallback?: (results: ShaderCompileResult[]) => void): void;
    getProgram(shaderCacheId: string): GLProgram | null;
    addMarcosHeader(header: string, shader: string, type: ShaderType, downgrade?: boolean): string;
    addShader(shader: ShaderWithSource): string;
    compileShader(shaderCacheId: string, asyncCallback?: (result: ShaderCompileResult) => void): ShaderCompileResult;
    createProgram(gl: WebGLRenderingContext, vs: string, fs: string, result: GLShaderCompileResult): () => (WebGLProgram | null);
    createShader(gl: WebGLRenderingContext, shaderType: GLenum, code: string): WebGLShader | null;
    deleteShader(cacheId: string): void;
    destroy(): void;
}
export {};
