import type { TextureConfigOptions, TextureOptions, TextureSourceOptions } from '../../types/Texture';
import type { GLGPURenderer } from './GLGPURenderer';
import type { TypedArray, vec2 } from '../../types/type';
type HTMLImageLike = ImageBitmap | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement;
type GLContext = WebGLRenderingContext | WebGL2RenderingContext;
export declare const imageBitMapAvailable: boolean;
export declare class GLTexture {
    readonly renderer?: GLGPURenderer;
    readonly glHandle: WebGLTexture | null;
    readonly options: TextureOptions;
    readonly target: GLenum;
    readonly gpuInfo: [format: number, type: number, target: number, mipmaps: vec2[]];
    width: number;
    height: number;
    _mipmapUsed: boolean;
    constructor(option: TextureOptions, renderer: GLGPURenderer);
    setInspectorName(name: string): void;
    errorLog(): void;
    update(source: TextureSourceOptions): void;
    setTextureFilters(gl: GLContext, target: GLenum, options: TextureConfigOptions): void;
    texImage2D(gl: GLContext, target: GLenum, level: number, internalformat: GLenum, format: GLenum, type: GLenum, image: HTMLImageLike): vec2;
    flipImageBitmap(bitmap: ImageBitmap): HTMLCanvasElement;
    texImage2DData(gl: GLContext, target: GLenum, level: number, internalformat: GLenum, format: GLenum, type: GLenum, data: {
        data: TypedArray;
        width: number;
        height: number;
    }): vec2;
    resizeImage(image: HTMLImageLike, targetWidth?: number, targetHeight?: number): HTMLCanvasElement | HTMLImageElement;
    offloadData(): void;
    destroy(): void;
    _assignRenderer(renderer: GLGPURenderer): void;
}
export declare function nearestPowerOfTwo(value: number): number;
export declare function isPowerOfTwo(value: number): boolean;
export {};
