import type { GLGPURenderer } from './GLGPURenderer';
import type { TypedArray, UniformValue, vec2 } from '../../types';
import { GLGPUBuffer } from './GLGPUBuffer';
export interface UniformBlockSpec {
    index: number;
    usedByVertexShader: boolean;
    usedByFragmentShader: boolean;
    size: number;
    uniformIndices: number[];
    used: boolean;
    name: string;
    uniforms: Record<string, BlockUniformInfo>;
    id: string;
}
type BlockUniformInfo = [
    type: number,
    offset: number,
    size: number,
    blockIndex: number,
    arrayStride: number,
    maxStride: number,
    rowMajor: number,
    index: number,
    byteLength: number
];
export declare function createUniformBlockSpecsFromProgram(gl: WebGL2RenderingContext, program: WebGLProgram, blockUniformNames: string[]): UniformBlockSpec[];
interface UBODirtyFlag {
    start: number;
    dirty: boolean;
    buffer?: TypedArray;
}
export declare class UniformBlockBuffer {
    buffer?: GLGPUBuffer;
    info: UniformBlockSpec;
    dirtyFlags: Record<string, UBODirtyFlag>;
    keepData: boolean;
    constructor(renderer: GLGPURenderer, info: UniformBlockSpec);
    setValues(uniformValues: Record<string, UniformValue>, dirtyFlags: Record<string, boolean>, uniformValueOffsets: Record<string, vec2>): void;
    bind(gl: WebGL2RenderingContext, program: WebGLProgram, bufferBindIndex: number): void;
    destroy(): void;
}
export {};
