import type { Material, MaterialDestroyOptions, MaterialOptions } from './Material';
import type { Geometry, GeometryOptions } from './Geometry';
import type { IGPUResource, IGPURenderer } from './IGPURenderer';
import type { GPURenderer } from './Renderer';
import type { DestroyOptions } from './constants';
interface MeshOptionsBase {
    material: Material | MaterialOptions;
    name?: string;
    priority?: number;
}
export interface GeometryMeshOptions extends MeshOptionsBase {
    geometry: Geometry | GeometryOptions;
}
export interface GeometriesMeshOptions extends MeshOptionsBase {
    geometries: Array<Geometry | GeometryOptions>;
}
export type MeshOptions = GeometryMeshOptions | GeometriesMeshOptions;
export type MeshDestroyOptions = {
    geometries?: DestroyOptions;
    material?: MaterialDestroyOptions | DestroyOptions.keep;
};
export declare class Mesh implements IGPUResource {
    readonly renderer?: IGPURenderer;
    readonly isDestroyed: boolean;
    readonly name: string;
    readonly geometries: Geometry[];
    readonly geometry?: Geometry;
    readonly material: Material;
    hide: boolean;
    priority: number;
    constructor(options: MeshOptions, renderer?: IGPURenderer);
    /**
     * update geometries
     * @param geos geometries to update
     * @param destroyGeometries whether to destroy previous geometries,default is true
     */
    setGeometries(geos: Geometry[], destroyGeometries?: DestroyOptions): void;
    /**
     * update material
     * @param mtl material to update
     * @param destroyMtl whether to destroy previous material, set true equals MaterialDestroyOptions { all:true }
     */
    setMaterial(mtl: Material | MaterialOptions, destroyMtl?: MaterialDestroyOptions | DestroyOptions.keep): void;
    destroy(options?: MeshDestroyOptions): void;
    assignRenderer(renderer: GPURenderer): Mesh;
}
export interface InstancedGeometryMeshOptions extends GeometryMeshOptions {
    instanceCount: number;
}
export interface InstancedGeometriesMeshOptions extends GeometriesMeshOptions {
    instanceCount: number;
}
export type InstancedMeshOptions = InstancedGeometryMeshOptions | InstancedGeometriesMeshOptions;
export declare class InstancedMesh extends Mesh {
    instanceCount: number;
    constructor(options: InstancedMeshOptions, renderer?: IGPURenderer);
}
export {};
