import type { ResourceInternal } from './ResourceInternal';
import type { TypedArray } from '../type';
import type { GPUBufferOptionsMemoryShared } from '../GPUBuffer';
export interface GPUBufferInternalOptions {
    data?: TypedArray;
    options?: typeof GPUBufferOptionsMemoryShared;
}
/**
 * 内存写入GPU的具体时机由框架自行决定，只要保证在渲染前输入GPU即可
 */
export declare abstract class GPUBufferInternal implements ResourceInternal {
    readonly isDestroyed: boolean;
    readonly byteLength: number;
    protected constructor(options: GPUBufferInternalOptions);
    abstract destroy(): void;
    /**
       * 整段替换数据，替换后byteLength会发生改变
       * 内存写入GPU的具体时机由框架自行决定，只要保证在渲染前输入GPU即可
       * 数据输入GPU后，可以丢弃内存端的数据
       * @param typedArray
       */
    abstract bufferData(typedArray: TypedArray): void;
    /**
       * 部分替换数据，如果offset+data的长度大于目前的byteLength，需要重新申请新的内存，整段替换
       * @param byteOffset 指针的offset，无类型区分
       * @param data
       */
    abstract bufferSubData(byteOffset: number, data: TypedArray): void;
    /**
       * 读取GPU数据，当创建参数有 GPUBufferOptionsMemoryShared 时，必须能读取到数据
       * 当框架没有读取到数据时，返回false
       * @param typedArray 将数据写入的arrayBuffer
       * @param offset 写入的指针偏移
       */
    abstract readSubData(typedArray: TypedArray, offset?: number): boolean;
}
