import type { TypedArray } from './type';
import type { IGPUResource, IGPURenderer } from './IGPURenderer';
import type { GPURenderer } from './Renderer';
export const GPUBufferOptionsMemoryShared = 1 << 1;
export type GPUBufferTarget =
  | WebGLRenderingContext['ARRAY_BUFFER']
  | WebGLRenderingContext['ELEMENT_ARRAY_BUFFER']
  | WebGL2RenderingContext['COPY_READ_BUFFER']
  | WebGL2RenderingContext['COPY_WRITE_BUFFER']
  | WebGL2RenderingContext['TRANSFORM_FEEDBACK_BUFFER']
  | WebGL2RenderingContext['UNIFORM_BUFFER']
  | WebGL2RenderingContext['PIXEL_PACK_BUFFER']
  | WebGL2RenderingContext['PIXEL_UNPACK_BUFFER'];

export interface GPUBufferOptions {
  name?: string,
  /**
   * 数据长度，如果提供了data，忽略此参数
   */
  byteLength?: number,

  data?: TypedArray,

  /**
   * GL需要设置，Vulkan和Metal此属性忽略
   */
  target?: GPUBufferTarget,

  /**
   * GL需要设置，Vulkan和Metal此属性忽略
   */
  usage?: WebGLRenderingContext['STATIC_DRAW'] | WebGLRenderingContext['DYNAMIC_DRAW'],
}

/**
 * GPU的Buffer对象，通过JS ArrayBuffer，可以向其中写入数据
 * JS调用写入数据后，会立即执行GPU写入的操作，此类型不会保存JS的ArrayBuffer对象
 *
 */
export declare class GPUBuffer implements IGPUResource {
  readonly renderer?: IGPURenderer;

  readonly isDestroyed: boolean;

  readonly level: number;

  readonly byteLength: number;

  readonly target: GPUBufferTarget;

  readonly usage?: WebGLRenderingContext['STATIC_DRAW'] | WebGLRenderingContext['DYNAMIC_DRAW'];

  constructor (options: GPUBufferOptions, renderer?: IGPURenderer);

  getElementCount (bytePerElement: number): number;
  /**
   * 根据TypedArray容量，重新分配Buffer内存
   * @param typedArray
   */
  bufferData (typedArray: TypedArray): void;

  /**
   * 更新buffer数据,立即写入GPU中
   * @param elementOffset 从buffer的第几个元素开始
   * @param typedArray 要写入的数据
   */

  bufferSubData (elementOffset: number, typedArray: TypedArray): void;

  /**
   * 将数据从GPU中读出，写入到传入的typedArray中，只有当level为2时才能成功，webgl下此函数永远不会成功
   * @param elementOffset 从第几个元素开始拷贝，默认为 0
   * @param typedArray 要写入的typedArray
   * @param elementCount 要写入多少个元素，如果不传入，则为typedArray元素长度
   */
  readSubData (elementOffset: number, typedArray: TypedArray, elementCount?: number): boolean;

  assignRenderer (renderer: GPURenderer): GPUBuffer;

  destroy (): void;
}

