import type { Immutable } from './type';
export const COMPRESSED_TEXTURE_NONE = 0;
export const COMPRESSED_TEXTURE_PVRTC = 1;
export const COMPRESSED_TEXTURE_ASTC = 2;

export interface GPUCapabilityDetail {
  maxSample: number,
  floatTexture: number,
  halfFloatTexture: number,
  //set currentPass color attachment type FLOAT
  floatColorAttachment?: boolean,
  //set currentPass color attachment type HALF_FLOAT
  halfFloatColorAttachment?: boolean,
  maxVertexUniforms: number,
  maxVertexTextures: number,
  maxFragmentUniforms: number,
  maxFragmentTextures: number,
  maxShaderTexCount: number,
  maxTextureSize: number,
  maxTextureAnisotropy: number,
  compressedTexture: number,
  shaderTextureLod: boolean,
  instanceDraw?: boolean,
  drawBuffers?: boolean,
  asyncShaderCompile?: boolean,
  //draw elements use uint32 Array
  intIndexElementBuffer?: boolean,
  //render pass depth and stencil texture readable
  //in webgl, if not readable,use RenderBuffer which cannot readable
  readableDepthStencilTextures?: boolean,
  writableFragDepth?: boolean,
}

export type GPUFramework = 'webgl' | 'webgl2' | 'opengles3' | 'opengles2' | 'node-gl' | 'metal' | 'none';

export interface GPUCapability {
  type: GPUFramework,
  level: number,
  capability: Immutable<GPUCapabilityDetail>,

  setup (gl: WebGLRenderingContext): void,
}

export class GPUCapabilityEmpty implements GPUCapability {
  type: GPUFramework;
  level = 0;
  capability = {
    maxSample: 0,
    floatTexture: 0,
    halfFloatTexture: 0,
    maxVertexUniforms: 0,
    maxVertexTextures: 0,
    maxFragmentUniforms: 0,
    maxFragmentTextures: 0,
    maxTextureSize: 0,
    maxShaderTexCount:0,
    maxTextureAnisotropy: 0,
    compressedTexture: 0,
    shaderTextureLod: false,
  };

  constructor () {
    this.type = 'none';
  }

  setup (gl: WebGLRenderingContext) {
  }
}
