import type {
  RenderPass,
  RenderPassLoadAction,
  RenderPassDestroyOptions, SemanticMap, SemanticGetter,
} from './RenderPass';
import type { Mesh } from './Mesh';
import type { vec4 } from './type';
import type { GPURenderer } from './index';
import type { IGPURenderer, IGPUResource } from './IGPURenderer';
import type { DestroyOptions } from './constants';
import type { InstancedMesh } from './Mesh';

export interface RenderState {
  currentMesh: Mesh | InstancedMesh,
  currentFrame: RenderFrame,
  currentPass: RenderPass,
}

export interface RenderFrameOptions {
  renderPasses?: RenderPass[],
  viewport?: vec4,
  semantics?: { [key: string]: SemanticGetter },
  loadAction?: RenderPassLoadAction,
  name?: string,
}

export type SemanticFunc = (state: RenderState) => SemanticGetter | null | undefined;

export type RenderFrameDestroyOptions = {
  passes?: RenderPassDestroyOptions | DestroyOptions.keep,
  semantics?: DestroyOptions,
};

export declare class RenderFrame implements IGPUResource {

  public loadAction: RenderPassLoadAction;

  constructor (options: RenderFrameOptions, renderer?: GPURenderer);

  readonly renderPasses: RenderPass[];

  readonly viewport?: vec4;

  readonly semantics: SemanticMap;

  readonly renderer?: IGPURenderer;

  readonly isDestroyed: boolean;

  setRenderPasses (passes: RenderPass[]): void;

  addRenderPass (pass: RenderPass): void;

  removeRenderPass (pass: RenderPass): void;

  render (): void;

  destroy (options?: RenderFrameDestroyOptions): void;

  assignRenderer (renderer: IGPURenderer): RenderFrame ;
}

/**
 * RenderFrame 渲染顺序如下：
 * 1. renderFrame 按照priority由底到高渲染renderPass，renderPass如果有设置attachments,则会创建frameBuffer对象
 * 2. 每个renderPass渲染时，按照meshes数组顺序进行渲染，renderPass可以制定meshes数组的排序方式，对每个mesh的priority进行排序
 * 3. 遍历Mesh时会将其Material中的Shader添加至shaderLibrary，再调用render之前，将所有shader进行编译
 */
