import type { MaterialInternal } from './MaterialInternal';
import type { vec2 } from '../type';
import type { TextureInternal } from './TextureInternal';
import type { RenderPassInternal } from './RenderPassInternal';

export interface TextureGpuInfo {
  format: number,
  type: number,
  target: number,
  mipmaps: vec2[],
  handle: number,
}

export abstract class AndGLContext {
  abstract isEnabled (param: GLenum): boolean;

  /**
   * 获取gl的参数， 返回值为4个int 数组
   * @param param
   */
  abstract getParameter_i4 (param: GLenum): [number, number, number, number];

  /**
   * 获取gl的参数， 返回值为4个bool 数组
   * @param param
   */
  abstract getParameter_b4 (param: GLenum): [boolean, boolean, boolean, boolean];

  /**
   * 获取gl的参数， 返回值为1个int
   * @param param
   */
  abstract getParameter_i1 (param: GLenum): number;

  abstract getParameter_f4 (param: GLenum): [number, number, number, number];

  abstract getParameter_f2 (param: GLenum): [number, number];

  /**
   * 将Material的状态设置到GPU
   * 这个方法会在Native端调用， 在RI中不会调用
   * 为了Android 测试，会主动调用设置状态到GPU, IOS 暂不调用
   */
  abstract setupStates (mtl: MaterialInternal): void;

  abstract getTextureGpuInfo (texture: TextureInternal): TextureGpuInfo;

  abstract getTexParameter (target: GLenum, pname: GLenum): number;

  abstract bindRenderPass (renderPass: RenderPassInternal): void;
}
