import type { ResourceInternal } from './ResourceInternal';
import type { TypedArray } from '../type';
import type { TextureInternal } from './TextureInternal';

export interface DataBlockInternalOptions {
  name: string,
}

/**
 * struct uniform的数据结构
 */
export interface NativeUniformStruct { [key: string]: TypedArray }

export abstract class DataBlockInternal implements ResourceInternal {
  readonly isDestroyed: boolean;
  /**
   * UBO的name，如果没有找到对应的UBO就用setUniform
   * UBO的Layout只有一种，如果不同shader中使用同名UBO，请创建不同的DataBlock对象
   */
  readonly name: string;

  protected constructor (options: DataBlockInternalOptions) {}

  abstract destroy (): void;

  /**
   * uniform 的数据都以TypedArray形式传入，
   * 在OpenGL中，使用UBO时，作为UBO的数据buffer，
   * 不使用UBO时根据反射类型进行数据转化
   * Metal中，直接当作 argument buffer
   */
  abstract setUniformValue (name: string, value: TypedArray): void;

  /**
   * 将uniform的数据拷贝到指定的TypedArray中,
   * 如果传入的TypedArray buffer 长度不足，将只写入传入长度的数据
   * @param name
   * @param value
   */
  abstract getUniformValue (name: string, value: TypedArray): boolean;

  /**
   * 设置uniform的结构体数据，对于数据进行深拷贝，修改原始数据，不会影响已经传入的数据，
   * 如果要修改数据，必须重新调用setUniformStructValues
   */
  abstract setUniformStructValues (name: string, values: NativeUniformStruct[]): boolean;

  /**
   * 获取uniform的结构体数据,获取的数据进行深拷贝,传入的数据将
   */
  abstract getUniformStructValue (name: string, idx: number): Record<string, ArrayBuffer> | undefined;

  abstract setUniformTextures (uniformName: string, texture: TextureInternal[] | undefined): void;

  abstract getUniformTextures (uniformName: string): TextureInternal[] | undefined;

}
