import type { AttributeWithType, GeometryOptions } from '../Geometry';
import type { ResourceInternal } from './ResourceInternal';
import type { GPUBufferInternal } from './GPUBufferInternal';

export interface GeometryInternalOptions extends Omit<GeometryOptions, 'attributes' | 'index'> {
  attributes: Record<string, AttributeWithType>,
  buffers: Record<string, GPUBufferInternal>,
  indexBuffer?: GPUBufferInternal,
  indexBufferType?: IndexBufferType,
}

export type IndexBufferType = WebGLRenderingContext['UNSIGNED_SHORT'] | WebGLRenderingContext['UNSIGNED_INT'];

/**
 * geometry中的数据都以buffer形式保存
 */
export abstract class GeometryInternal implements ResourceInternal {
  readonly isDestroyed: boolean;

  readonly name: string;

  drawStart: number;

  drawCount: number;

  mode: GLenum;

  protected constructor (options: GeometryInternalOptions) {
  }

  abstract setAttributeBuffer (name: string, buffer: GPUBufferInternal | undefined): void;

  abstract getAttributeBuffer (name: string): GPUBufferInternal | undefined;

  abstract getIndexBuffer (): [buffer: GPUBufferInternal | undefined, type: number];

  /**
     * 获得当前所有的attributeBuffer
     */
  abstract getAttributeBufferMap (): Record<string, GPUBufferInternal>;

  abstract setIndexBuffer (buffer: GPUBufferInternal | undefined, bufferType?: IndexBufferType): void;

  /**
     * 并不会销毁attributeBuffers和IndexBuffer，
     * 但会清除内部buffer的引用，如果需要销毁buffers，
     * 调用destroy之前，需要手动调用 getAttributeBufferMap 和 getIndexBuffer 将获得的buffer显式销毁
     */
  abstract destroy (): void;
}
