import type { ResourceInternal } from './ResourceInternal';
import type { RenderPassInternal } from './RenderPassInternal';
import type { RenderPassLoadAction } from '../RenderPass';

export interface RenderFrameInternalOptions {
  renderPasses: RenderPassInternal[],
  loadAction: RenderPassLoadAction,
  name: string,
}
export abstract class RenderFrameInternal implements ResourceInternal {
  readonly isDestroyed: boolean;
  readonly loadAction: RenderPassLoadAction;
  readonly name: string;
  /**
   * 增删改查的操作，在warp层对renderPassInternals进行后，得到的数据全部覆盖当前数据
   */
  renderPasses: RenderPassInternal[];

  protected constructor (options: RenderFrameInternalOptions) {
  }
  abstract destroy (): void;

  /**
   * 代表此帧会在渲染队列的下一帧被渲染，调用此函数，也会调用renderer.invalid方法
   * 由于渲染是异步的，此函数在Metal/Vulkan实现中，并不会直接绘制
   */
  abstract render (): void;

  abstract updateLoadAction (action: RenderPassLoadAction): void;
}
