import type { ResourceInternal } from './ResourceInternal';
import type { MeshInternal } from './MeshInternal';
import type { RenderPassLoadAction, RenderPassStoreAction } from '../RenderPass';
import type { TextureInternal } from './TextureInternal';
import type { RenderPassStorageObject } from '../RenderPass';
import type { RenderPassAttachmentStorageType } from '../RenderPass';

//--- color ---
export interface RenderPassColorAttachmentInternal {
  readonly storageType: RenderPassAttachmentStorageType.color,
  readonly texture: TextureInternal,
}

//--- depth stencil ---
export interface RenderPassDepthStencilAttachmentInternal {
  readonly storageType: RenderPassAttachmentStorageType,
  readonly texture?: TextureInternal,
  readonly storage?: RenderPassStorageObjectInternal,
}
/**
 * 在Metal里面，所有的RenderPassAttachment都以Texture存储
 * 在GL中，可以使用RenderTarget/Texture作为fbo的attachment
 */
export interface RenderPassStorageObjectInternal extends RenderPassStorageObject {
  _mtlTexture?: TextureInternal,
  _glRenderBuffer?: WebGLRenderbuffer,
}

export interface RenderPassDepthStencilAttachmentOptionsInternal {
  storageType: RenderPassAttachmentStorageType,
  texture?: TextureInternal,
  storage?: RenderPassStorageObjectInternal,
}

export interface RenderPassOptionsInternal {
  loadAction: RenderPassLoadAction,
  attachments: RenderPassColorAttachmentInternal[],
  name: string,
  meshes: MeshInternal[],
  width: number,
  height: number,
  multiSample?: number,
  storeAction?: RenderPassStoreAction,
  depthStencilAttachment?: RenderPassDepthStencilAttachmentOptionsInternal,
}

export abstract class RenderPassInternal implements ResourceInternal {
  readonly isDestroyed: boolean;
  readonly name: string;
  readonly storeAction: RenderPassStoreAction;
  readonly loadAction: RenderPassLoadAction;
  readonly multisample: number;
  readonly depthStencilAttachment: RenderPassDepthStencilAttachmentInternal;
  readonly attachments: RenderPassColorAttachmentInternal[];
  readonly width: number;
  readonly height: number;
  /**
   * 增删改查的操作，在warp层对Mesh进行后，得到的数据全部覆盖当前数据
   */
  meshes: MeshInternal[];

  priority: number;

  protected constructor (options: RenderPassOptionsInternal) {
  }
  abstract destroy (): void;

  abstract resize (width: number, height: number): void;
}
