import type { TextureFormatOptions, TextureConfigOptionsBase, ImageData } from '../Texture';
import type { ResourceInternal } from './ResourceInternal';
import type { ImageBitmapInternal } from './ImageBitmapInternal';

export interface TextureInternalOptions extends TextureFormatOptions, Omit<TextureConfigOptionsBase, 'keepImageSource'> {
  target: WebGLRenderingContext['TEXTURE_CUBE_MAP'] | WebGLRenderingContext['TEXTURE_2D'],
  mipmapCount?: number,
}

export interface TextureInternalConstructOptions extends TextureInternalOptions {
  width: number,
  height: number,
}

export type TextureInternalCubeImages<T extends ImageBitmapInternal | ImageData = ImageBitmapInternal> = [
    positiveX: T,
    nagtiveX: T,
    positiveY: T,
    nagtiveY: T,
    positiveZ: T,
    nagtiveZ: T
];

export abstract class TextureInternal implements ResourceInternal {
  readonly isDestroyed: boolean;

  readonly width: number;

  readonly height: number;

  readonly name: string;

  readonly options: TextureInternalOptions;

  readonly error?: number;

  protected constructor (options: TextureInternalConstructOptions) {
  }

  abstract resize (width: number, height: number): boolean;

  abstract destroy (): void;

  /**
     * 上传cubeImage解码后的数据
     */
  abstract updateCubeImagesData (images: TextureInternalCubeImages<ImageData>, mipmapLevel?: number): void;

  /**
     * 上传cubeImage
     */
  abstract updateCubeImages (images: TextureInternalCubeImages, mipmapLevel?: number): void;

  /**
     * 上传整张解码后图片数据,数据尺寸必须和Texture尺寸定义相同
     */
  abstract update2DImageData (data: ImageData, mipmapLevel?: number): void;

  /**
     * 上传2D压缩纹理数据
     */
  abstract update2DCompressedData (data: Uint8Array, mipmapLevel?: number): void;

  /**
     * 上传Cube压缩纹理数据
     */
  abstract update2DCompressedCubeData (data: Uint8Array[], mipmapLevel?: number): void;

  /**
     * imageBitmap的尺寸也必须和Texture相同
     * @param image
     * @param mipmapLevel
     */
  abstract update2DImage (image: ImageBitmapInternal, mipmapLevel?: number): void;

}
